@@ -845,5 +845,11 @@ got=$(f 2>&1)
 	diff -U1 <(print "$exp") <(print "$got") | sed $'1,3 d; s,^,\t,' >&2
 }
 
+# ======
+got=$("$SHELL" -c '(typeset -T echo=(typeset TYPEVAR); echo V; print $V); echo OK' 2>&1)
+exp=$'( typeset TYPEVAR )\nOK'
+[[ $got == "$exp" ]] || err_exit 'type definition overriding regular built-in' \
+	"(expected $(printf %q "$exp"), got $(printf %q "$got"))"
+
 # ======
 exit $((Errors<125?Errors:125))
