@@ -342,7 +342,6 @@ extern char*		fmttime(const char*, time_t);
 extern char*		fmtuid(int);
 extern char*		fmtversion(unsigned long);
 extern void*		memdup(const void*, size_t);
-extern void		memfatal(void);
 extern unsigned int	memhash(const void*, int);
 extern unsigned long	memsum(const void*, int, unsigned long);
 extern char*		pathaccess(char*, const char*, const char*, const char*, int);
@@ -365,8 +364,6 @@ extern size_t		pathnative(const char*, char*, size_t);
 extern char*		pathpath(char*, const char*, const char*, int);
 extern char*		pathpath_20100601(const char*, const char*, int, char*, size_t);
 extern size_t		pathposix(const char*, char*, size_t);
-extern char*		pathprobe(char*, char*, const char*, const char*, const char*, int);
-extern char*		pathprobe_20100601(const char*, const char*, const char*, int, char*, size_t, char*, size_t);
 extern size_t		pathprog(const char*, char*, size_t);
 extern char*		pathrepl(char*, const char*, const char*);
 extern char*		pathrepl_20100601(char*, size_t, const char*, const char*);
@@ -428,22 +425,6 @@ extern int		wc2utf8(char*, uint32_t);
 extern char**		environ;
 #endif
 
-/*
- * really handy malloc()/free() (__FILE__,__LINE__,__FUNCTION__) tracing
- * make with VMDEBUG==1 or debug=1 or CCFLAGS=$(CC.DEBUG)
- * VMDEBUG==0 disables
- * at runtime export VMALLOC_OPTIONS per vmalloc.3
- * to list originating call locations
- */
-
-#if !_std_malloc && !defined(VMFL) && !defined(_VMHDR_H) && \
-	(VMDEBUG || !defined(VMDEBUG) && _BLD_DEBUG)
-
-#define VMFL	1
-#include <vmalloc.h>
-
-#endif
-
 #include <ast_api.h>
 
 #undef	AST_PLUGIN_VERSION
